/*
 * Copyright (C) 2012, Eric Wong <normalperson@yhbt.net>
 * License: GPLv3 or later (see COPYING for details)
 */
#ifdef HAVE_OPEN_MEMSTREAM
# define my_memstream_close(fp,dst,bytes) fclose((fp))
#else
# define open_memstream(dst,bytes) tmpfile()
static int my_memstream_close(FILE *fp, char **dst, size_t *bytes)
{
	long pos;

	errno = 0;
	pos = ftell(fp);

	if (pos >= 0) {
		rewind(fp);
		*bytes = (size_t)pos;
		*dst = xmalloc(*bytes);
		if (fread(*dst, 1, *bytes, fp) == *bytes)
			return fclose(fp);
	} else {
		assert(errno != EBADF
		       && "EBADF in stdio/open_memstream replacement");
	}

	*bytes = 0;
	syslog(LOG_ERR, "stdio/open_memstream replacement failed: %m");

	/*
	 * if ftell() fails, fclose() may fail due to an I/O error,
	 * too, but at least hope we release memory..
	 */
	(void)fclose(fp);

	return 0;
}
#endif /* !HAVE_OPEN_MEMSTREAM */
